<?php

namespace Controller;

use Application;
use Boutique;
use Cms;
use Commande;
use Breadcrumb;
use Exception;
use Form\ContactForm;
use GroupePrix;
use Mailer;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Texte;
use Tools;

/**
 * Class ContactController
 * @package Controller
 */
class ContactController extends Controller
{

    /**
     * Constructor
     *
     * @param Request $request the current request
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);
    }


    public function setup()
    {
        $flashbag = $this->getFlashBag();
        $post = [];
        if (!$flashbag->has("post") && $cli = $this->getCurrentClient()) {
            $reqPost = $this->request->attributes->all(); // bring back given info, eg: origin
//            dump($cli);
            $post = array_merge($reqPost, array(
                "client_prenom" => $cli->client_prenom,
                "client_nom"    => $cli->client_nom,
                "client_email"  => $cli->client_email,
                'client_societe' => $cli->client_societe,
                'client_telephone' => $cli->adresse_telephone,
            ));
        }

        $cms = new Cms();
        $arrSujet = $cms->getSujet();
        $this->template->assign("arrSujet", $arrSujet);

        if ($this->request->get("sujet_id")) {
            $post["sujet_id"] = $this->request->get("sujet_id");
        }

        // recup commande courante
        if ($cust = $this->getCurrentClient()) {
            $cmd = new Commande();
            $arrCommande = $cmd->get(array("client_id" => $cust->client_id));
            $this->template->assign("arrCommande", $arrCommande);
        }

        $flashbag->set("post", array_merge($post, $flashbag->get("post")));

        $keys = array("error", "success", "post");
        foreach ($keys as $key) {
            if ($flashbag->has($key)) {
                $this->template->assign($key, $flashbag->get($key));
            }
        }
    }

    /**
     * @Route("/contact.html", name="contact", methods = { "GET" }, options={"_sitemap" = {}})
     */
    public function contact()
    {
        $this->setup();

        $bc = Breadcrumb::getInstance();
        $bc->add("Contact" ,$this->generateUrl('contact'));

        return $this->render('contact.tpl');
    }

    public function show()
    {
        if ($this->request->get("origin") == "compte_sav") {
            $this->template->assign("isAccountSav", true);
        }

        $this->setup();

        return $this->fetch('form/contact.tpl');
    }

    /**
     * @Route("/contact", name="contact_post", methods = { "POST" })
     */
    public function post()
    {
        $req = $this->request->request;
        $flashbag = $this->getFlashBag();
        $texte = new Texte();
        $form = new ContactForm();

        $error = $form->handle($this->request);

        if (!empty($error)) {

            $json['field_errors'] = json_encode($error);

        } else {
            $mailer = new Mailer(Application::getEmailSender());
            $mailCode = "contact";

            try {
                // defaults
                $expected = array(
                    "client_email",
                    "client_nom",
                    "client_prenom",
                    "client_telephone",
                    "sujet_id",
                    "client_message",
                );
                $tags = array_combine($expected, array_fill(0, count($expected), ""));

                // merge with posted
                $tags = array_merge($tags, $req->all());

                // sujet
                $emailRecipient = Boutique::getInstance()->contactEmail;
                if ($req->has("sujet_id")) {
                    $cms = new Cms();
                    $sujet = $cms->getSujet(["sujet_id" => $req->get("sujet_id")]);
                    $tags = array_merge($tags, $sujet[0]);
                    $emailRecipient = $sujet[0]["sujet_nom_email"]; // if sujet id, change recipient
                }


                $mailer->send($mailCode, array($emailRecipient), $tags);
                $flashbag->add("success", $texte->getOneOrCreate(
                    "succMailSent",
                    "Votre message a bien été pris en compte."
                ));
            } catch (Exception $e) {
                $flashbag->add("error", $texte->getOneOrCreate(
                    "errSendMail",
                    "Une erreur s'est produite et a empêché l'envoi d'email, merci de ré-essayer plus tard."
                ));

            }

            $json['redirect'] = $this->generateUrl("contact");

            return $this->response($json);
        }

        return $this->response($json);
    }
}
