<?php

namespace EventListener;

use Event\TemplateEvent;
use Referencement;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class ReferencementListener
 * @package EventListener
 */
class ReferencementListener implements EventSubscriberInterface
{
    /**
     * @param TemplateEvent $event
     * @throws \Exception
     */
    public function onPreRender(TemplateEvent $event)
    {
        $request = $event->getRequest();
        $engine = $event->getEngine();

        $referencement = new Referencement();
        $formvars = array();
        $nfoUrl = $request->getPathInfo();

        $formvars['referencement_url'] = addslashes($nfoUrl);
        $arrRef = $referencement->get($formvars);
        $ref = array();
        if(!empty($arrRef[0]['referencement_meta_title'])) $ref['title'] = $arrRef[0]['referencement_meta_title'];
        if(!empty($arrRef[0]['referencement_meta_desc'])) $ref['description'] = $arrRef[0]['referencement_meta_desc'];
        if(!empty($arrRef[0]['referencement_meta_kw'])) $ref['keywords'] = $arrRef[0]['referencement_meta_kw'];
        if(!empty($arrRef[0]['referencement_meta_other'])) $ref['other'] = $arrRef[0]['referencement_meta_other'];
        $engine->assign('superRef',$ref);
    }

    /**
     * Returns an array of event names this subscriber wants to listen to.
     *
     * The array keys are event names and the value can be:
     *
     *  * The method name to call (priority defaults to 0)
     *  * An array composed of the method name to call and the priority
     *  * An array of arrays composed of the method names to call and respective
     *    priorities, or 0 if unset
     *
     * For instance:
     *
     *  * array('eventName' => 'methodName')
     *  * array('eventName' => array('methodName', $priority))
     *  * array('eventName' => array(array('methodName1', $priority), array('methodName2')))
     *
     * @return array The event names to listen to
     */
    public static function getSubscribedEvents()
    {
        return [
            TemplateEvent::PRE_RENDER => array(array('onPreRender', 32)),
        ];
    }
}