<?php

namespace Shopbydedi;

use Controller\OrderController;
use Event\CommandeEvent;
use Event\CommandeStatutEvent;
use Model;
use Boutique;
use DediException;
use Livraison;
use Application;
use Query;
use Parrainage;
use ClubFidelite;
use Stock;
use Symfony\Component\Routing\Router;
use Texte;
use Exception;
use Client;
use Adresse;
use Pays;
use EmailSender;

/**
 * Class Commande
 * @package Shopbydedi
 */
class Commande extends Model {
    /**
     * Represent status commande : en attente de paiement
     * commande passé mais paiement non validé
     */
    const STATUT_ATTENTE = 1;
    const STATUT_PAIEMENT = 2;
    const STATUT_PREPARATION = 3;
    const STATUT_EXPEDIE = 4;
    const STATUT_RELANCE = 98;
    const STATUT_ANNULEE = 99;

    /**
     * @var string
     */
    protected $table = 'commande';

    public function getOne($formvars = array())
    {
        $result = $this->get($formvars);
        return (!empty($result)) ? $result[0] : array();
    }

    /**
     * Cherche des commandes avec les filtres passé en paramètre
     *
     * @param array  $formvars tableau d'option :
     *  - commande_id : pour remonter une commande précise
     *  - onlyColissimo : les commandes avec Colissimo pour mode de livraison
     *  - produit_nom : commande avec un produit dont le nom est passé en paramètre
     *  - client_id : rechercher les commandes d'un client pércis
     * @param string $champs   a préciser si on veut juste remonter
     * la valeur d'un champ
     *
     * @return string|array    Retourne la valeur du champ spécifié
     * dans le paramètres $champs, sinon un tableau avec les résultats,
     * chaque ligne represente une commande
     */
    function get($formvars = array(), $champs = "")
    {

        $sql = "SELECT c.*, csn.*";
        $sql .=", DATE_FORMAT(commande_date_ajout,'%d/%m/%Y') AS commande_date";
        $sql .=", DATE_FORMAT(commande_facture_date,'%d/%m/%Y') AS commande_facture_date";
        $sql.=", c.commande_stotal_ht AS stotal_ht";
        $sql.=", c.commande_stotal_ttc AS stotal_ttc";
        $sql.=", c.commande_total_ht AS total_ht";
        $sql.=", c.commande_total_ttc AS total_ttc";

        // Gestion paiement
        $sql.=", ROUND((SELECT SUM(cpapp.commande_paiement_montant) FROM commande_paiement cpapp WHERE cpapp.commande_paiement_statut_id = 1 AND c.commande_id = cpapp.commande_id),2) AS montant_a_paye";
        $sql.=", ROUND((SELECT SUM(cpapp.commande_paiement_montant) FROM commande_paiement cpapp WHERE cpapp.commande_paiement_statut_id = 2 AND c.commande_id = cpapp.commande_id),2) AS montant_paye";

        $sql.=", paien.paiement_nom_nom, paien.paiement_nom_description, paien.paiement_nom_txt_template, paien.paiement_nom_txt_email";
        $sql.=", c.commande_id";

        $sql.=" FROM (commande c)";
        $sql.=" INNER JOIN client cl ON(c.client_id = cl.client_id)";
        $sql.=" LEFT JOIN commande_produit cp ON (cp.commande_id = c.commande_id)";
        $sql.=" LEFT JOIN commande_produit_attribut cpa ON (cp.commande_produit_id = cpa.commande_produit_id)";
        $sql.=" LEFT JOIN commande_statut cs ON (cs.commande_statut_id = c.commande_statut_id)";
        $sql.=" LEFT JOIN commande_statut_nom csn ON (csn.commande_statut_id = cs.commande_statut_id AND csn.langue_id = c.langue_id)";
        $sql.=" LEFT JOIN commande_paiement cpaie ON(c.commande_id = cpaie.commande_id)";
        $sql.=" LEFT JOIN paiement_nom paien ON(cpaie.paiement_id = paien.paiement_id AND paien.langue_id = c.langue_id)";

        $sql.=" WHERE 1";
        if (empty($formvars['no_boutique'])) {
            $sql.=" AND cl.boutique_id = '".Boutique::getInstance()->id."'";
        }

        if (!empty($formvars['commande_id']))
            $sql.=" AND c.commande_id = '" . $formvars['commande_id'] . "'";
        if (!empty($formvars['c.commande_id']))
            $sql.=" AND c.commande_id = '" . $formvars['c.commande_id'] . "'";
        if (!empty($formvars['onlyColissimo']))
            $sql.=" AND c.commande_livraison_code IN ('COL','COLD','CDS','COLI')";
        if (!empty($formvars['list_commande_livraison_code']))
            $sql.=" AND c.commande_livraison_code IN (" . $formvars['list_commande_livraison_code'] . ")";
        if (!empty($formvars['commande_livraison_code'])) {
            if (is_array($formvars['commande_livraison_code'])) {
                $sql.=" AND c.commande_livraison_code IN ('" . implode("','", $formvars['commande_livraison_code']) . "')";
            } else {
                $sql .= " AND c.commande_livraison_code = '" . $formvars['commande_livraison_code'] . "'";
            }
        }
        if (!empty($formvars['produit_nom']))
            $sql.=" AND cp.produit_nom = '" . $formvars['produit_nom'] . "'";
        if (!empty($formvars['produit_ref']))
            $sql.=" AND cp.produit_ref = '" . $formvars['produit_ref'] . "'";
        if (!empty($formvars['client_id']))
            $sql.=" AND c.client_id = '" . $formvars['client_id'] . "'";
        if (!empty($formvars['commande_statut_id']))
            $sql.=" AND c.commande_statut_id = '" . $formvars['commande_statut_id'] . "'";
        if (!empty($formvars['list_commande_statut_id']))
            $sql.=" AND c.commande_statut_id IN (" . $formvars['list_commande_statut_id'] . ")";
        if (!empty($formvars['withFacture']))
            $sql.=" AND c.commande_facture_numero != '0'";
        if (!empty($formvars['withoutTracking']))
            $sql.=" AND c.commande_livraison_colis = ''";
        if (!empty($formvars['paiement_id']))
            $sql.=" AND c.paiement_id = '" . $formvars['paiement_id'] . "'";
        if (!empty($formvars['delai_relance']))
            $sql.=" AND DATE_ADD(DATE(commande_date_ajout), INTERVAL " . $formvars['delai_relance'] . " DAY) = CURRENT_DATE()";

        if (!empty($formvars['facture_date_debut'])) {
            list($j, $m, $a) = explode("/", $formvars['facture_date_debut']);
            $sql.=" AND DATE(c.commande_facture_date) >= '" . $a . "-" . $m . "-" . $j . "'";
        }
        if (!empty($formvars['facture_date_fin'])) {
            list($j, $m, $a) = explode("/", $formvars['facture_date_fin']);
            $sql.=" AND DATE(c.commande_facture_date) <= '" . $a . "-" . $m . "-" . $j . "'";
        }

        if (!empty($formvars['withTracking'])) {
            $sql.=" AND c.commande_livraison_colis <> ''";
        }

        $sql.=" GROUP BY c.commande_id";

        if (!empty($formvars['orderasc'])) {
            $sql .= " ORDER BY commande_date_ajout ASC";
        } else {
            $sql .= " ORDER BY commande_date_ajout DESC";
        }

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     *  recupere l'historique de statut d'une commande
     *
     * @param array  $formvars
     * @param string $champs
     *
     * @return array|string
     */
    function getHistory($formvars = array(), $champs = "")
    {

        $sql = " SELECT *, DATE_FORMAT(commande_statut_historique_date_ajout,'%d/%m/%Y') AS commande_date";
        $sql.=" FROM commande_statut_historique csh ";
        $sql.=" LEFT JOIN commande_statut_nom csn on (csn.commande_statut_id = csh.commande_statut_id AND csn.langue_id = '" . Boutique::getInstance()->getLangue()->id . "') ";
        $sql.="LEFT JOIN commande_statut cs on (cs.commande_statut_id = csh.commande_statut_id)";
        $sql.=" WHERE 1";

        if (!empty($formvars['commande_id'])) {
            $sql .= " AND csh.commande_id = '" . $formvars['commande_id'] . "'";
        }
        if (!empty($formvars['commande_statut_id'])) {
            $sql .= " AND csh.commande_statut_id = '" . $formvars['commande_statut_id'] . "'";
        }
        $sql.=" ORDER BY commande_statut_historique_date_ajout DESC";

        if (empty($champs)) {
//            dump($sql);die;
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     * @param array $formvars
     * @param string $champs
     * @return null
     *
     */
    function getProduit($formvars = array(), $champs = "")
    {

        $sql = "SELECT cp.*, cpa.*, pt.*, cp.commande_produit_id";
//        $sql = "SELECT cp.*";
//        $sql.= ", m.marque_nom";

        // info de qte et d'achat
        $sql.=", IF(commande_produit_attribut_id IS NULL,cp.produit_quantite,produit_option_valeur_quantite) AS qte";
        $sql.=", IF(commande_produit_attribut_id IS NULL,cp.produit_achat,produit_option_valeur_achat) AS achat_ht";
        $sql.=", IF(commande_produit_attribut_id IS NULL,cp.produit_ref,produit_attribut_ref) AS produit_ref";
        $sql.=", IF(commande_produit_attribut_id IS NULL,cp.produit_ref_frs,produit_attribut_ref_frs) AS produit_ref_frs";
        $sql.=", ROUND(IF(commande_produit_attribut_id IS NULL,cp.produit_prix,+produit_option_valeur_prix),2) AS prix_ht";
        $sql.=", ROUND(IF(commande_produit_attribut_id IS NULL,cp.produit_total_ht,((produit_option_valeur_prix)*produit_option_valeur_quantite)),2) AS prix_total_ht";
        $sql.=", IF(commande_produit_attribut_id IS NULL,cp.produit_prix_ttc, produit_option_valeur_prix_ttc) AS prix_ttc";
        $sql.=", IF(commande_produit_attribut_id IS NULL,cp.produit_total_ttc,(produit_option_valeur_prix_ttc)*produit_option_valeur_quantite) AS prix_total_ttc";

        $sql.=" FROM (commande_produit cp)";
        $sql.=" LEFT JOIN commande_produit_attribut cpa ON (cp.commande_produit_id = cpa.commande_produit_id)";
        $sql.=" LEFT JOIN produit_type pt ON (cp.produit_type_id = pt.produit_type_id)";
        $sql.=" LEFT JOIN produit p ON (cp.produit_id = p.produit_id)";
        $sql.=" LEFT JOIN marque m ON (p.marque_id = m.marque_id)";
//
        $sql.=" WHERE 1";

        if (!empty($formvars['commande_id'])) {
            $sql .= " AND cp.commande_id = '" . $formvars['commande_id'] . "'";
        } elseif(!empty($this->commande_id)) {
            $sql .= " AND cp.commande_id = '" . $this->commande_id . "'";
        }

//        if (!empty($formvars['commande_produit_attribut_id']))
//            $sql.=" AND cpa.commande_produit_attribut_id = '" . $formvars['commande_produit_attribut_id'] . "'";
//        dump($this->sql->getAll($sql));die;
//        dump($sql);die;
        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    /**
     * ajoute une commande avec ses produits / modes de paiements / mode de livraison
     *
     * @param \Panier $panier objet panier avec
     * toutes les infos de la commande a créer
     *
     * @return integer
     * @throws DediException
     */
    function saveFromPanier(\Panier $panier)
    {
        $customer = $panier->getCustomer();
        $addr = array();
        $addr['livraison'] = $panier->getAddressShipping();
        $addr['facturation'] = $panier->getAddressBilling();
        $shipping = $panier->getShipping();
        $isTva = $panier->isTva();
        $relay = $panier->getRelay();
        $payments = $panier->getPayment();
        $delai = $panier->getDelaiExpedie();

        if (empty($customer)) {
            throw new DediException('no customer');
        }

        if (empty($addr['livraison'])) {
            throw new DediException('no address shipping');
        }

        if (empty($addr['facturation'])) {
            throw new DediException('no address billing');
        }

        if (empty($shipping)) {
            throw new DediException('no shipping');
        }

        if (Livraison::isRelay($shipping) && empty($relay)) {
            throw new DediException('no relay');
        }

        $data = array();
        $data['commande_livraison_delai'] = (!empty($delai)) ? $delai : 1;

        // totaux pre calculé
        list($data['commande_stotal_ht'], $data['commande_stotal_ttc']) = $panier->getTotalProducts();
        list($data['commande_total_ht'], $data['commande_total_ttc']) = $panier->getTotalProductsAndShip();
        $data['commande_stotal_ttc'] = ($isTva) ? $data['commande_stotal_ttc']:$data['commande_stotal_ht'];
        $data['commande_total_ttc'] = ($isTva) ? $data['commande_total_ttc']:$data['commande_total_ht'];
        $data['commande_stotal_taxe'] = ($data['commande_stotal_ttc'] - $data['commande_stotal_ht']);
        $data['commande_total_taxe'] = ($data['commande_total_ttc'] - $data['commande_total_ht']);

        $data['commande_statut_id'] = (!empty($payments[0]->commande_statut_id)) ? $payments[0]->commande_statut_id : Commande::STATUT_ATTENTE;

        $data['commande_poids'] = $panier->getWeight();

        if (Livraison::isRelay($shipping)) {
            $data['relais_id'] = $relay->id;
        }

        $data['commande_device'] = $panier->getDeviceType();

        $this->sql->beginTransaction();

        $idCommande = $this->create($customer, $addr['facturation'], $addr['livraison'], $shipping, $data);

        if (empty($idCommande)) {

            $this->sql->db->rollback();
            $this->sql->endTransaction();

            throw new DediException(
                'error add commande - request : ' . $this->sql->db->error
            );
        }

        // on stock le referer
        // @TODO : souci ici
        // $tracking = new Tracking;
        // $tracking->setReferer('commande', $idCommande, true);

        // ajout des produits
        $this->addProduit($idCommande, $panier->getProducts(), $isTva);

        // ajout des paiements
        $this->addPaiement($idCommande, $payments);

        $reduction = $panier->getReduction();
        if ($reduction !== null) {
            // ajout des code promos / reduction
            $this->addReduction($idCommande, $reduction);

            // met a jour les utilisations des codes promos
            $this->usePromo($customer->client_id, $reduction);
        }

        // ajout des bon d'achats
        $this->addBonAchat($idCommande, $panier->getBonAchat());

        // on ajoute l'historique de statut
        $this->changeStatut($idCommande, $data['commande_statut_id'], true, array('bc' => true), 0);

        $this->sql->endTransaction();

        Application::getEventDispatcher()->dispatch(
            CommandeEvent::ADD, new CommandeEvent(new Commande($idCommande))
        );

        return $idCommande;
    }

    /**
     * Créer une entrée dans la table commande, retourne l'id commande généré
     *
     * @param Client $customer
     * @param Adresse $addrBilling
     * @param Adresse $addrShipping
     * @param Livraison $shipping
     * @param array $data
     *
     * @return int
     *
     * @throws DediException
     */
    public function create(Client $customer, Adresse $addrBilling, Adresse $addrShipping, Livraison $shipping, $data = array())
    {
        $addr = [
            'livraison' => $addrShipping,
            'facturation' => $addrBilling
        ];

        $data = array_merge($data, $customer->toArray(), $shipping->toArray());

        $data['client_civilite'] = $customer->civilite_nom_nom;
        $data['client_rue'] = $customer->adresse_rue;
        $data['client_rue2'] = $customer->adresse_rue2;
        $data['client_ville'] = $customer->adresse_ville;
        $data['client_cp'] = $customer->adresse_cp;
        $data['client_pays'] = $customer->pays_nom_nom;

        $typeAddr = array('livraison', 'facturation');
        foreach ($typeAddr as $ta) {
            $data[$ta . '_civilite'] = $addr[$ta]->civilite_nom_nom;
            $data[$ta . '_nom'] = $addr[$ta]->adresse_nom;
            $data[$ta . '_prenom'] = $addr[$ta]->adresse_prenom;
            $data[$ta . '_societe'] = $addr[$ta]->adresse_societe;
            $data[$ta . '_rue'] = $addr[$ta]->adresse_rue;
            $data[$ta . '_rue2'] = $addr[$ta]->adresse_rue2;
            $data[$ta . '_ville'] = $addr[$ta]->adresse_ville;
            $data[$ta . '_cp'] = $addr[$ta]->adresse_cp;
            $data[$ta . '_pays'] = $addr[$ta]->pays_nom_nom;
            $data[$ta . '_pays_code'] = $addr[$ta]->pays_iso_code_2;
            $data[$ta . '_pays_id'] = $addr[$ta]->pays_id;
            $data[$ta . '_telephone'] = $addr[$ta]->adresse_telephone;
        }

        $isTva = (new Pays($addr['livraison']->pays_id))->isTva();

        $data['commande_port_ht'] = $shipping->port_ht;
        $data['commande_port_ttc'] = $shipping->port_ttc;
        $data['commande_port_taxe'] = ($isTva) ? $shipping->taxe_taux : 0;
        $data['commande_livraison_nom'] = $shipping->livraison_nom_nom;
        $data['commande_livraison_code'] = $shipping->livraison_code;
        $data['commande_livraison_id'] = $shipping->livraison_id;

        $data['commande_livraison_delai_date_min'] = date('Y-m-d', $shipping->delai_date_min);
        $data['commande_livraison_delai_date_max'] = date('Y-m-d', $shipping->delai_date_max);

        $data['langue_id'] = Boutique::getInstance()->getLangue()->id;

        return $this->save($data);
    }

    /**
     * Ajoute les produits passé en paramètres pour une commande
     *
     * @param integer $idCommande
     * @param array   $products
     * @param boolean $isTva
     *
     * @return void
     * @throws DediException
     */
    function addProduit($idCommande, $products, $isTva = true) {

        $query = new Query();

        foreach($products as $product) {

            $data = $product;

            if (!empty($product['produit_attribut_id'])) {
                $query->reset()->select()
                    ->from('commande_produit')
                    ->where('produit_id', $product['produit_id'])
                    ->where('commande_id', $idCommande);
                $lineProd = $this->sql->getOne($query->build());
                if (!empty($lineProd)) {
                    $data['commande_produit_id'] = $lineProd['commande_produit_id'];
                }
            }

            $data['commande_id'] = $idCommande;

            $data['produit_prix_ttc'] = ($isTva) ? round($product['produit_prix_ht'] * (1 + $product['taxe_taux']), 2) : round($product['produit_prix_ht'], 2);
            $data['produit_prix_taxe'] = $data['produit_prix_ttc'] - $product['produit_prix_ht'];
            $data['produit_total_ht'] = $product['produit_prix_ht'] * $product['quantite_order'];
            $data['produit_total_ttc'] = $data['produit_prix_ttc'] * $product['quantite_order'];
            $data['produit_total_taxe'] = $data['produit_total_ttc'] - $data['produit_total_ht'];

            $data['produit_quantite'] = $product['quantite_order'];
            $data['produit_nom'] = $product['produit_nom_nom'];
            $data['produit_taxe'] = $product['taxe_taux'];
            $data['produit_prix'] = $product['produit_prix_ht'];

            if (empty($data['commande_produit_id'])) {
                $data['commande_produit_id'] = $this->sql->save('commande_produit', $data);

                if(empty($data['commande_produit_id'])) {
                    $this->sql->db->rollback();
                    $this->sql->endTransaction();

                    throw new DediException(
                        'error add product commande - request : ' . $this->sql->db->error
                    );
                }
            } else {
                $query->reset()->update('commande_produit')
                    ->value('produit_quantite', $lineProd['produit_quantite'] + $product['quantite_order'])
                    ->value('produit_total_ht', $lineProd['produit_total_ht'] + $data['produit_total_ht'])
                    ->value('produit_total_ttc', $lineProd['produit_total_ttc'] + $data['produit_total_ttc'])
                    ->value('produit_total_taxe', $lineProd['produit_total_taxe'] + $data['produit_total_taxe'])
                    ->where('commande_produit_id', $data['commande_produit_id']);

                $this->sql->query($query->build());
            }

            if (!empty($data['produit_attribut_id'])) {
                $data['produit_option_valeur_prix'] = $product['attribut_complement_produit_prix_ht'];
                $data['produit_option_valeur_prix_ttc'] = $product['attribut_complement_produit_prix_ttc'];
                $data['produit_option_valeur_prix_taxe'] = $data['produit_option_valeur_prix_ttc'] - $data['produit_option_valeur_prix'];
                $data['produit_option_valeur_total_ht'] = $data['produit_option_valeur_prix'] * $product['quantite_order'];
                $data['produit_option_valeur_total_ttc'] = $data['produit_option_valeur_prix_ttc'] * $product['quantite_order'];
                $data['produit_option_valeur_total_taxe'] = $data['produit_option_valeur_total_ttc'] - $data['produit_option_valeur_total_ht'];

                $data['produit_option_valeur_quantite'] = $product['quantite_order'];

                $result = $this->sql->save('commande_produit_attribut', $data);

                if(empty($result)) {
                    $this->sql->db->rollback();
                    $this->sql->endTransaction();

                    throw new DediException(
                        'error add product attr commande - request : ' . $this->sql->db->error
                    );
                }
            }
        }
    }

    /**
     * Ajout d'un statut
     *
     * @param $idCommande
     * @param $idStatut
     *
     * @return void
     * @throws DediException
     */
    function addStatut($idCommande, $idStatut) {

        $this->sql->beginTransaction();

        $data =  array('commande_id' => $idCommande, 'commande_statut_id' => $idStatut);

        $result = $this->sql->save('commande_statut_historique', $data);

        if(empty($result)) {
            $this->sql->db->rollback();
            $this->sql->endTransaction();

            throw new DediException(
                'error add statut commande - request : ' . $this->sql->db->error
            );
        }

        $result = $this->sql->save('commande', $data);

        if(empty($result)) {
            $this->sql->db->rollback();
            $this->sql->endTransaction();

            throw new DediException(
                'error update statut commande - request : ' . $this->sql->db->error
            );
        }

        $this->sql->endTransaction();
    }

    /**
     * @param integer    $idCommande
     * @param \Paiement[] $payments
     *
     * @return void
     * @throws DediException
     */
    function addPaiement($idCommande, $payments) {

        // @TODO : a finir

        foreach ($payments as $pay) {

            $data = array();
            $data['commande_id'] = $idCommande;
            $data['paiement_id'] = $pay->paiement_id;
            $data['commande_paiement_montant'] = $pay->paiement_montant;
            $data['commande_paiement_nom'] = $pay->paiement_nom_nom;
            $data['commande_paiement_statut_id'] = 1;

            $result = $this->sql->save('commande_paiement', $data);

            if(empty($result)) {
                $this->sql->db->rollback();
                $this->sql->endTransaction();

                throw new DediException(
                    'error add paiement - request : ' . $this->sql->db->error
                );
            }

            // on passe le statut du paiement a utiliser si besoin
            /*
            if (!empty($pay->paiement_class)) {
                // @TODO : a revoir avec interface
                $class = new $pay->paiement_class;
                $class->usePaiement($pay->paiement_ref_id);
            }
            */
        }
    }

    /**
     * active tous les moyens de paiement, lors du passage en paiement autorise
     *
     * @param $idCommande
     */
    function activePaiement($idCommande) {

        $sql = "UPDATE commande_paiement SET commande_paiement_statut_id = 2, commande_paiement_date_modif = NOW() WHERE commande_id = '" . $idCommande . "'";
        $this->sql->query($sql);
    }

    /**
     * @param $idCommande
     * @param $reductions \Reduction
     *
     * @throws DediException
     */
    public function addReduction($idCommande, $reduction)
    {
        $data['commande_id'] = $idCommande;
        $data['produit_prix'] = -$reduction->price;
        $data['produit_nom'] = $reduction->name;
        $data['produit_ref'] = $reduction->code;
        $data['produit_quantite'] = 1;

        $data['produit_prix_ttc'] = $data['produit_prix'];
        $data['produit_total_ht'] = $data['produit_prix'] * $data['produit_quantite'];
        $data['produit_total_ttc'] = $data['produit_prix_ttc'] * $data['produit_quantite'];

        $result = $this->sql->save('commande_produit', $data);

        if (empty($result)) {
            $this->sql->db->rollback();
            $this->sql->endTransaction();

            throw new DediException(
                'error add reduction - request : ' . $this->sql->db->error
            );
        }
    }

    /**
     * @param $idCommande
     * @param $bonAchats \BonAchat[]
     *
     * @throws DediException
     */
    public function addBonAchat($idCommande, $bonAchats)
    {
        foreach ($bonAchats as $bonAchat) {

            $data = array();
            $data['commande_id'] = $idCommande;
            $data['commande_paiement_montant'] = $bonAchat->bon_achat_prix;
            $data['commande_paiement_nom'] = $bonAchat->bon_achat_nom;
            $data['commande_paiement_statut_id'] = 2;
            $data['commande_paiement_code'] = $bonAchat->bon_achat_id;
            $data['commande_paiement_type'] = 'ba';
            $data['commande_paiement_operator'] = -1;

            $result = $this->sql->save('commande_paiement', $data);

            if (empty($result)) {
                $this->sql->db->rollback();
                $this->sql->endTransaction();

                throw new DediException(
                    'error add bon achat - request : ' . $this->sql->db->error
                );
            } else {
                // on utilise le BA comme moyen de paiement => passage au statut utilisé
                $result = $bonAchat->useToPay();
                if ($result == 0) {
                    throw new DediException('Error when use to pay bon achat');
                }
            }
        }
    }

    /**
     * @param $idCommande
     * @param $newStatut
     * @param bool $envoiMail
     * @param array $tags
     * @param string $oldStatut
     */
    function changeStatut($idCommande, $newStatut, $envoiMail = false, $tags = array(), $oldStatut = "") {

        // si pas de statut on le recupere
        if ($oldStatut === "") {
            $sql = "SELECT commande_statut_id FROM commande WHERE commande_id = '" . $idCommande . "'";
            $result = $this->sql->getOne($sql);
            $oldStatut = $result['commande_statut_id'];
        }

        if ($oldStatut != $newStatut) {

            if (!empty($newStatut)) {
                // on met le statut a jour
                $this->addStatut($idCommande, $newStatut);
            }

            // on recupere le necessaire pour savoir si on met les stocks a jour ?
            $stock = new Stock;
            $arrMvtStatut = $stock->initMvt(); // on recup le tableau
            if (isset($arrMvtStatut[$oldStatut])) {
                $mvtInfo = $arrMvtStatut[$oldStatut][$newStatut][0];
                $mvtPhys = $arrMvtStatut[$oldStatut][$newStatut][1];
            } else {
                $statutValide = array();
                foreach ($arrMvtStatut as $key => $value) {
                    if (!empty($key)) {
                        array_push($statutValide, $key);
                    }
                }
                $sql = "SELECT commande_statut_id FROM commande_statut_historique WHERE commande_id = '" . $idCommande . "' AND commande_statut_id IN (" . implode(",", $statutValide) . ")ORDER BY commande_statut_historique_date_ajout DESC LIMIT 1,1";
                $result = $this->sql->getOne($sql);
                $oldOldStatut = $result['commande_statut_id'];
                if (empty($oldOldStatut))
                    $oldOldStatut = 0;
                $mvtInfo = $arrMvtStatut[$oldOldStatut][$newStatut][0];
                $mvtPhys = $arrMvtStatut[$oldOldStatut][$newStatut][1];
            }

            if (!empty($mvtInfo) || !empty($mvtPhys)) {
                $arrProduit = $this->getProduit(array('commande_id' => $idCommande));
                foreach ($arrProduit as $produit) {
                    if ($produit['produit_type_stock'] == 1) {
                        $stock->mouvement($produit['produit_id'], $produit['produit_attribut_id'], $idCommande, "", $produit['qte'], $produit['achat_ht'], $mvtInfo, $mvtPhys);
                    }
                }
            }

            // action par statut
            switch ($newStatut) {
                // paiement autorise
                case self::STATUT_ATTENTE:
                    $envoiMailAdmin = 1;
                    break;
                // paiement autorise
                case self::STATUT_PAIEMENT:
                    // on recupere le nombre de statut expedie dans l'histo de la commande
                    $arrPay = $this->getHistory(array('commande_id' => $idCommande, 'commande_statut_id' => self::STATUT_PAIEMENT));
                    if (count($arrPay) == 1) {
                        // activation des paiements
                        $this->activePaiement($idCommande);
                    }
                    $envoiMailAdmin = 1;
                    break;
                // expedition de la commande
                case self::STATUT_EXPEDIE:
                    // on recupere le nombre de statut expedie dans l'histo de la commande
                    $arrExpe = $this->getHistory(array('commande_id' => $idCommande, 'commande_statut_id' => self::STATUT_EXPEDIE));
                    if (count($arrExpe) == 1) {
                        // creation de la facture
                        $this->createFacture($idCommande);
                        // parrainage
                        $parrainage = new Parrainage;
                        $parrainage->add($idCommande);

                        $clubFidelite = new ClubFidelite();
                        $clubFidelite->add($idCommande);
                    }
                    break;

                default:
                    break;
            }

            //Verifie si le produit_type demande l'utilisation d'une fonction
            $arrProduit = $this->getProduit(array('commande_id' => $idCommande));
            foreach ($arrProduit as $produit) {
                if (!empty($produit['produit_type_class'])) {
                    $class = new $produit['produit_type_class'];
                    if (method_exists($class, 'changeStatut')) {
                        $class->changeStatut($idCommande, $newStatut, $produit);
                    }
                }
            }
        }

        // Gestion du numero de suivi
        if (!empty($tags['colis'])) {
            $sql = "UPDATE commande SET commande_livraison_colis = '" . $tags['colis'] . "' WHERE commande_id = '" . $idCommande . "'";
            $this->sql->query($sql);
        }
        // envoi du mail
        if ($envoiMail == true) {

            list($arrCommande) = $this->get(array('commande_id' => $idCommande));

            $siteUrl = SITE_URL;

            if (empty($tags['from_bo'])) {
                $siteUrl = rtrim($siteUrl, '/');
            }

            $siteUrl = \Tools::transformUrlWithAuthHttp($siteUrl);

            $arrFile = array();
            // envoie du bon de commande
            if (!empty($tags['bc'])) {
                $param = array("commande_id" => $idCommande, "client_id" => $arrCommande['client_id']);

                if (!empty($tags['from_bo'])) {
                    $path = Application::getRouter()->getRouter()->generate("bon_commande", $param, Router::RELATIVE_PATH);
                } else {
                    $path = Application::getRouter()->getRouter()->generate("bon_commande", $param);
                }

                $arrFile[] = array($siteUrl . $path, "bc.pdf");
            }

            // envoie de la facture
            if (!empty($tags['fc'])) {
                $param = array("commande_id" => $idCommande, "client_id" => $arrCommande['client_id']);

                if (!empty($tags['from_bo'])) {
                    $path = Application::getRouter()->getRouter()->generate("facture", $param, Router::RELATIVE_PATH);
                } else {
                    $path = Application::getRouter()->getRouter()->generate("facture", $param);
                }
                $arrFile[] = array($siteUrl . $path, "fc.pdf");
            }

            list($sujet, $txt, $html) = $this->getMail($idCommande, $newStatut, $tags);

            $to = array($arrCommande['client_email'] => $arrCommande['client_prenom'] . ' ' . $arrCommande['client_nom']);

            if (!empty($envoiMailAdmin)) {
                $boutique = Boutique::getInstance();
                $to[$boutique->contactEmail] = $boutique->nom;
            }

            Application::getEmailSender()->send(EmailSender::createMessage($to, $sujet, $html, null, $arrFile));
        }

        $orderEvent = new \Commande($idCommande);

        // event changement de statut
        Application::getEventDispatcher()->dispatch(
            CommandeStatutEvent::CHANGE,
            new CommandeStatutEvent($orderEvent, $oldStatut, $newStatut)
        );

        // event specs sur certains statut
        if (!empty($eventToLaunch)) {
            Application::getEventDispatcher()->dispatch(
                $eventToLaunch,
                new CommandeStatutEvent($orderEvent, $oldStatut, $newStatut)
            );
        }
    }

    /**
     * @param $idClient
     * @param \Reduction $reduction
     */
    public function usePromo($idClient, $reduction)
    {
        $sql = "
                UPDATE promotion 
                SET promotion_nombre_utilisation = promotion_nombre_utilisation - 1 
                WHERE promotion_id = '" . $reduction->id . "' AND promotion_nombre_utilisation > 0";
        $this->sql->query($sql);

        $sql = "INSERT INTO promotion_client SET promotion_id = '".$reduction->id."', client_id = '".$idClient."'";
        $this->sql->query($sql);
    }

    public function getPaiement()
    {
        $this->exceptionIfNotLoadCommande();

        $query = new Query();
        $query->select()->from('commande_paiement')->where('paiement_id IS NOT NULL')
            ->where('commande_id', $this->commande_id);
        return $this->sql->getAll($query->build());
    }

    public function getReduction()
    {
        $this->exceptionIfNotLoadCommande();

        $query = new Query();
        $query->select()->from('commande_produit')->where('produit_id IS NULL')
            ->where('produit_type_id IS NULL')
            ->where('produit_prix < 0')
            ->where('commande_id', $this->commande_id);
        return $this->sql->getAll($query->build());
    }

    public function getBonAchat()
    {
        $this->exceptionIfNotLoadCommande();

        $query = new Query();
        $query->select()->from('commande_paiement')->where('paiement_id IS NULL')
            ->where('commande_paiement_type', 'ba')
            ->where('commande_id', $this->commande_id);
        return $this->sql->getAll($query->build());
    }

    protected function exceptionIfNotLoadCommande()
    {
        if (empty($this->commande_id)) {
            throw new DediException('You need to load one "Commande"');
        }
    }

    /**
     * creation de la facture, si elle n'existe pas encore
     *
     * @param $idCommande
     */
    function createFacture($idCommande) {

        $sql = "SELECT MAX(commande_facture_numero) as numero FROM commande";
        $result = $this->sql->getOne($sql);
        $numFacture = $result['numero'];

        $sql = "UPDATE commande SET";
        $sql.=" commande_facture_numero = " . $numFacture . " + 1";
        $sql.=", commande_facture_date = NOW()";
        $sql.=" WHERE commande_id = '" . $idCommande . "' AND commande_facture_numero=''";
        $this->sql->query($sql);
    }

    /**
     * @param $idCommande
     * @param $idStatut
     * @param array $tags
     * @return array
     */
    function getMail($idCommande, $idStatut, $tags = array()) {

        $sql = "SELECT csn.*";
        $sql.=" FROM (commande_statut_nom csn, commande c)";
        $sql.=" WHERE csn.langue_id = c.langue_id";
        $sql.=" AND c.commande_id = '" . $idCommande . "'";
        $sql.=" AND csn.commande_statut_id = '" . $idStatut . "'";
        $arrStatut = $this->sql->getOne($sql);

        list($arrCommande) = $this->get(array('commande_id' => $idCommande));

        $sujet = $arrStatut['commande_statut_nom_sujet'];
        $txt = $arrStatut['commande_statut_nom_email'];
        $html = $arrStatut['commande_statut_nom_email_html'];

        // TAG SPEC : ADRESSE_LIVRAISON
        $arrCommande['adresse_livraison'] = "";
        if (!empty($arrCommande['livraison_societe']))
            $arrCommande['adresse_livraison'].=$arrCommande['livraison_societe'] . "<br/>";
        $arrCommande['adresse_livraison'].=$arrCommande['livraison_prenom'] . ' ' . $arrCommande['livraison_nom'] . "<br/>";
        $arrCommande['adresse_livraison'].=$arrCommande['livraison_rue'] . "<br/>";
        if (!empty($arrCommande['livraison_rue2']))
            $arrCommande['adresse_livraison'].=$arrCommande['livraison_rue2'] . "<br/>";
        $arrCommande['adresse_livraison'].=$arrCommande['livraison_cp'] . " " . $arrCommande['livraison_ville'] . "<br/>";
        $arrCommande['adresse_livraison'].=$arrCommande['livraison_pays'];
        // TAG SPEC : ADRESSE_FACTURATION
        $arrCommande['adresse_facturation'] = "";
        if (!empty($arrCommande['facturation_societe']))
            $arrCommande['adresse_facturation'].=$arrCommande['facturation_societe'] . "<br/>";
        $arrCommande['adresse_facturation'].=$arrCommande['facturation_prenom'] . ' ' . $arrCommande['facturation_nom'] . "<br/>";
        $arrCommande['adresse_facturation'].=$arrCommande['facturation_rue'] . "<br/>";
        if (!empty($arrCommande['facturation_rue2']))
            $arrCommande['adresse_facturation'].=$arrCommande['facturation_rue2'] . "<br/>";
        $arrCommande['adresse_facturation'].=$arrCommande['facturation_cp'] . " " . $arrCommande['facturation_ville'] . "<br/>";
        $arrCommande['adresse_facturation'].=$arrCommande['facturation_pays'];
        // TAG SPEC : VAT
        if ($arrCommande['commande_taxe'] == 1)
            $arrCommande['vat'] = 'TTC';
        else
            $arrCommande['vat'] = 'HT';
        // TAG SPEC : LIVRAISON
        $livraison = new Livraison;
        list($arrLivraison) = $livraison->get(array('livraison_id' => $arrCommande['livraison_id']));
        $arrCommande = array_merge($arrCommande, $arrLivraison);
        // TAG SPEC : LISTE PRODUITS
        $arrCommande['liste_produits'] = "";
        $arrProduit = $this->getProduit(array('commande_id' => $idCommande));
        foreach ($arrProduit as $produit) {
            $arrCommande['liste_produits'].=$produit['qte'] . " x " . $produit['produit_ref'] . " - " . $produit['produit_nom'] . ' - '.$produit['marque_nom'];
            if (!empty($produit['produit_option_valeur_nom']))
                $arrCommande['liste_produits'].=" (" . $produit['produit_option_valeur_nom'] . ")";
            $arrCommande['liste_produits'].=" : " . $produit['prix_total_ttc'] . " Euro(s)<br/>";
        }

        $arrCommande['cp'] = $arrCommande['livraison_cp'];

        // recalcul des totaux
        $arrCommande['total_ttc']+= $arrCommande['commande_frais_assurance'] + $arrCommande['commande_frais_traitement'];

        if (empty($tags['colis']) && !empty($arrCommande['commande_livraison_colis'])) {
            $tags['colis'] = $arrCommande['commande_livraison_colis'];
        }

        // premiere passe de remplacement
        $texte = new Texte();
        $sujet = $texte->replaceTag($sujet, $arrCommande);
        $txt = $texte->replaceTag($txt, $arrCommande);
        $html = $texte->replaceTag($html, $arrCommande);

        // Tags sup
        if (!empty($tags)) {
            foreach ($tags as $key => $value) {
                $txt = $texte->replaceIfTag(mb_strtoupper($key), $value, $txt);
                $html = $texte->replaceIfTag(mb_strtoupper($key), $value, $html);
            }
        }

        return array($sujet, $txt, $html);
    }

    /**
     * Check si une commande a été crée dans les 2 dernieres minutes
     * @return bool
     */
    public function checkCommandePaiement()
    {

        if(!empty($_SESSION['commande_id'])) {
            $sql="SELECT commande_id";
            $sql.=" FROM commande";
            $sql.=" WHERE commande_id = '".$_SESSION['commande_id']."'";
            $sql.=" AND commande_date_ajout >= DATE_SUB(NOW(), INTERVAL 2 MINUTE)";
            $result = $this->sql->getOne($sql);

            return ($_SESSION['commande_id'] == $result['commande_id']);
        }

        return false;
    }
}